import unittest
import numpy
import gc
import copy

from dcmt import DcmtParameterError, DcmtError, DcmtRandom, DcmtRandomState, mt_range


def testLimits(randoms, start, stop):
	stop = float(stop)
	start = float(start)
	ideal_mean = (stop + start) / 2
	ideal_var = (((stop - start) / 2) ** 3 / 3 * 2) / (stop - start)

	return numpy.abs(randoms.mean() - ideal_mean) / (stop - start), \
		numpy.abs(randoms.var() - ideal_var) / ideal_var

def getRandomArray(rng, N):
	if isinstance(rng, DcmtRandom):
		return numpy.array([rng.random() for i in xrange(N)])
	else:
		return rng.rand(N)


class TestErrors(unittest.TestCase):

	def testWordlen(self):

		ids = (1,)
		tests = (
			(DcmtRandom, ()),
			(DcmtRandom.range, ids),
			(DcmtRandomState, ()),
			(DcmtRandomState.range, ids),
			(mt_range, ids)
		)

		for func, args in tests:

			# correct
			for wordlen in (31, 32):
				kwds = dict(wordlen=wordlen, gen_seed=1)
				func(*args, **kwds)

			# incorrect
			for wordlen in (16, 64):
				kwds = dict(wordlen=wordlen, gen_seed=1)
				self.assertRaises(DcmtParameterError, func, *args, **kwds)

	def testExponent(self):

		#correct_exponents = [521, 607, 1279, 2203, 2281, 3217, 4253, 4423,
		#	9689, 9941, 11213, 19937, 21701, 23209, 44497]

		# small list for testing purposes, since the whole list takes
		# ages to process
		correct_exponents_reduced = [521, 607]

		ids = (1,)
		tests = (
			(DcmtRandom, ()),
			(DcmtRandom.range, ids),
			(DcmtRandomState, ()),
			(DcmtRandomState.range, ids),
			(mt_range, ids)
		)

		for func, args in tests:

			# correct
			for exponent in correct_exponents_reduced:
				kwds = dict(exponent=exponent, gen_seed=1)
				func(*args, **kwds)

			# incorrect
			for exponent in (500, 1000, 50000):
				kwds = dict(exponent=exponent, gen_seed=1)
				self.assertRaises(DcmtParameterError, func, *args, **kwds)

	def testId(self):

		# valid calls
		DcmtRandom(id=20, gen_seed=1)
		DcmtRandom.range(1, 4, gen_seed=1)
		DcmtRandomState(id=3000, gen_seed=1)
		DcmtRandomState.range(100, 103, gen_seed=1)
		mt_range(4, gen_seed=1)

		# check that range functions behave like Python range()
		for func in (DcmtRandom.range, DcmtRandomState.range, mt_range):
			for args in ((), (1, 2, 3)):
				self.assertRaises(TypeError, func, *args)

			self.assertEqual(func(3, 1), [])

		# error: id < 0
		self.assertRaises(DcmtParameterError, DcmtRandom, id=-1, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandom.range, -1, 3, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandomState, id=-1, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandomState.range, -1, 2, gen_seed=1)
		self.assertRaises(DcmtParameterError, mt_range, -1, 3, gen_seed=1)

		# error: id > 65535
		self.assertRaises(DcmtParameterError, DcmtRandom, id=65536, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandom.range, 65535, 65537, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandomState, id=65536, gen_seed=1)
		self.assertRaises(DcmtParameterError, DcmtRandomState.range, 65535, 65537, gen_seed=1)
		self.assertRaises(DcmtParameterError, mt_range, 65534, 65539, gen_seed=1)
		self.assertRaises(DcmtParameterError, mt_range, 65536, 65539, gen_seed=1)

	def testBugId9(self):

		kwds = dict(wordlen=31, exponent=521, gen_seed=1)

		# known bug: cannot find generator for wordlen=31, exponent=521 and id=9
		self.assertRaises(DcmtParameterError, DcmtRandom, id=9, **kwds)
		self.assertRaises(DcmtParameterError, DcmtRandom.range, 8, 11, **kwds)
		self.assertRaises(DcmtParameterError, DcmtRandomState, id=9, **kwds)
		self.assertRaises(DcmtParameterError, DcmtRandomState.range, 8, 11, **kwds)
		self.assertRaises(DcmtParameterError, mt_range, 7, 11, **kwds)

	def testSeed(self):

		correct_seeds = (None, 0, 1, 2**16, long(2**16), 2**32 - 1, long(2**32 - 1),
			-1, -2**32 / 2, -2**32, 2**32, 2 * 2**32, long(2**32), (1, 2, 3))
		incorrect_seeds = ({}, [])

		ids = (1,)
		tests = (
			(DcmtRandom, (), lambda x: x),
			(DcmtRandom.range, ids, lambda x: x[0]),
			(DcmtRandomState, (), lambda x: x),
			(DcmtRandomState.range, ids, lambda x: x[0]),
			(mt_range, ids, None)
		)

		for func, args, get_rng in tests:

			# check that default (random) seed works
			rng = func(*args)

			if get_rng is not None:
				get_rng(rng).seed()

			# correct seeds
			for seed in correct_seeds:
				rng = func(*args, gen_seed=seed)

				if get_rng is not None:
					get_rng(rng).seed(seed)

			# incorrect seeds
			for seed in incorrect_seeds:
				self.assertRaises(DcmtParameterError, func, *args, gen_seed=seed)

				if get_rng is not None:
					rng = get_rng(func(*args, gen_seed=1))
					self.assertRaises(DcmtParameterError, rng.seed, seed)

	def testDefaultSeed(self):
		"""Check that RNG is useable even if seed was not set explicitly"""

		ids = (1,)
		tests = (
			(DcmtRandom, (), lambda x: x),
			(DcmtRandom.range, ids, lambda x: x[0]),
			(DcmtRandomState, (), lambda x: x),
			(DcmtRandomState.range, ids, lambda x: x[0]),
			(mt_range, ids, lambda x: DcmtRandomState.from_mt_range(*x)[0])
		)

		for func, args, get_rng in tests:
			rng = get_rng(func(*args, gen_seed=1))
			getRandomArray(rng, 10)

	def testPositionalSeed(self):
		"""Check that initializing seed can be given as a positional argument"""
		tests = (DcmtRandom, DcmtRandomState)

		for cls in tests:
			rng = cls(1, gen_seed=2)
			getRandomArray(rng, 10)


class TestBasics(unittest.TestCase):

	def testDifferentIds(self):

		for cls in (DcmtRandom, DcmtRandomState):

			rngs = cls.range(2, gen_seed=10)
			rngs[0].seed(2)
			rngs[1].seed(2)

			N = 10
			randoms0 = getRandomArray(rngs[0], N)
			randoms1 = getRandomArray(rngs[1], N)

			self.assert_((randoms0 != randoms1).all())

	def testDeterminism(self):
		gen_id = 10
		seed = 100
		init_seed = 300

		for cls in (DcmtRandom, DcmtRandomState):

			rng0 = cls(id=gen_id, gen_seed=seed)
			rng0.seed(init_seed)
			rng1 = cls(id=gen_id, gen_seed=seed)
			rng1.seed(init_seed)

			randoms0 = getRandomArray(rng0, 10)
			randoms1 = getRandomArray(rng1, 10)

			self.assert_((randoms0 == randoms1).all())

	def testRangeMethod(self):
		gen_id = 10
		seed = 100
		init_seed = 300
		N = 10

		for cls in (DcmtRandom, DcmtRandomState):
			rngs = cls.range(3, gen_seed=seed)

			for rng in rngs:
				rng.seed(init_seed)

			randoms0 = getRandomArray(rngs[0], N)
			randoms1 = getRandomArray(rngs[1], N)
			randoms2 = getRandomArray(rngs[2], N)

			self.assert_((randoms0 != randoms1).all())
			self.assert_((randoms0 != randoms2).all())

	def testRandomsRange(self):

		for cls in (DcmtRandom, DcmtRandomState):
			for wordlen in (31, 32):
				rng = cls(wordlen=wordlen, gen_seed=99)
				rng.seed(999)

				randoms = getRandomArray(rng, 1000)

				diff_mean, diff_var = testLimits(randoms, 0, 1)
				self.assert_(diff_mean < 0.05)
				self.assert_(diff_var < 0.05)

	def testState(self):

		tests = (
			(DcmtRandom, 'getstate', 'setstate'),
			(DcmtRandomState, 'get_state', 'set_state'),
		)

		for cls, getstate, setstate in tests:
			rng = cls(gen_seed=55)
			rng.seed(555)
			N = 10

			state = getattr(rng, getstate)() # getstate/setstate

			rng2 = copy.copy(rng) # copy

			# reference
			randoms0 = getRandomArray(rng, N)

			getattr(rng, setstate)(state)
			randoms1 = getRandomArray(rng, N)

			randoms2 = getRandomArray(rng2, N)

			self.assert_((randoms0 == randoms1).all())
			self.assert_((randoms0 == randoms2).all())


class TestRandom(unittest.TestCase):

	def testGetrandbits(self):
		rng = DcmtRandom(gen_seed=55)
		rng.seed(555)
		stop = 2 ** 64

		randoms = numpy.array([float(rng.randrange(0, stop)) for i in xrange(1000)])

		diff_mean, diff_var = testLimits(randoms, 0, stop)

		self.assert_(diff_mean < 0.05)
		self.assert_(diff_var < 0.05)


class TestRandomState(unittest.TestCase):

	def testRand(self):

		shape = (9, 10, 11)
		rng = DcmtRandomState(gen_seed=900)
		rng.seed(400)

		randoms = rng.rand(*shape)

		self.assert_(randoms.shape == shape)
		diff_mean, diff_var = testLimits(randoms, 0, 1)
		self.assert_(diff_mean < 0.05)
		self.assert_(diff_var < 0.05)

	def testMtRange(self):
		start = 0
		stop = 3
		seed = 100
		init_seeds = [4, 5, 6]
		N = 10

		mt_common, mt_unique = mt_range(start, stop, gen_seed=seed)
		rngs0 = DcmtRandomState.from_mt_range(mt_common, mt_unique)
		rngs1 = DcmtRandomState.range(start, stop, gen_seed=seed)

		for rng0, rng1, seed in zip(rngs0, rngs1, init_seeds):
			rng0.seed(seed)
			rng1.seed(seed)

		for rng0, rng1 in zip(rngs0, rngs1):
			randoms0 = getRandomArray(rng0, N)
			randoms1 = getRandomArray(rng1, N)
			self.assert_((randoms0 == randoms1).all())


def getLineCount(fname):
	f = open(fname)
	count = len(f.read().split('\n'))
	f.close()
	return count

if __name__ == '__main__':

	suites = []

	for cls in (
			TestErrors,
			TestBasics,
			TestRandom,
			TestRandomState,
		):
		suites.append(unittest.TestLoader().loadTestsFromTestCase(cls))

	all_tests = unittest.TestSuite(suites)
	unittest.TextTestRunner(verbosity=1).run(all_tests)

	gc.collect() # additional test for different pointer- and reference- related bugs

	# Temporary measure while I'm optimizing C/Python ratio in .pyx files
	cython_files = [
		'../src/wrapper/pyrandom.c',
		'../src/wrapper/numpyrandom.c'
	]
	for fname in cython_files:
		print "File: " + fname + ": " + str(getLineCount(fname)) + " lines"



include "common.pxi"

import numpy

from dcmt.exceptions import DcmtError, DcmtParameterError

cdef extern from "numpy/arrayobject.h":

	ctypedef int npy_intp

	ctypedef extern class numpy.dtype [object PyArray_Descr]:
		cdef int type_num, elsize, alignment
		cdef char type, kind, byteorder, hasobject
		cdef object fields, typeobj

	ctypedef extern class numpy.ndarray [object PyArrayObject]:
		cdef char *data
		cdef int nd
		cdef npy_intp *dimensions
		cdef npy_intp *strides
		cdef object base
		cdef dtype descr
		cdef int flags

	cdef enum NPY_TYPES:
		NPY_BOOL
		NPY_BYTE
		NPY_UBYTE
		NPY_SHORT
		NPY_USHORT
		NPY_INT
		NPY_UINT
		NPY_LONG
		NPY_ULONG
		NPY_LONGLONG
		NPY_ULONGLONG
		NPY_FLOAT
		NPY_DOUBLE
		NPY_LONGDOUBLE
		NPY_CFLOAT
		NPY_CDOUBLE
		NPY_CLONGDOUBLE
		NPY_OBJECT
		NPY_STRING
		NPY_UNICODE
		NPY_VOID
		NPY_NTYPES
		NPY_NOTYPE

	npy_intp PyArray_SIZE(ndarray arr)
	object PyArray_ContiguousFromObject(object obj, NPY_TYPES type,
		int mindim, int maxdim)

	void import_array()


# Initialize numpy
import_array()

cdef object uint32_to_python(uint32_t x):
	if x > 0x7FFFFFFF and INT_MAX < 0xFFFFFFFF:
		return PyLong_FromUnsignedLong(x)
	else:
		return PyInt_FromLong(x)

cdef uint32_t uint32_from_python(object x):
	return PyInt_AsUnsignedLongMask(x)

cdef get_mt_struct_fields(mt_struct *mt):
	return {
		'aaa': uint32_to_python(mt.aaa),
		'mm': int(mt.mm),
		'nn': int(mt.nn),
		'rr': int(mt.rr),
		'ww': int(mt.ww),
		'wmask': uint32_to_python(mt.wmask),
		'umask': uint32_to_python(mt.umask),
		'lmask': uint32_to_python(mt.lmask),
		'shift0': int(mt.shift0),
		'shift1': int(mt.shift1),
		'shiftB': int(mt.shiftB),
		'shiftC': int(mt.shiftC),
		'maskB': uint32_to_python(mt.maskB),
		'maskC': uint32_to_python(mt.maskC),
		'i': int(mt.i)
	}

cdef set_mt_struct_fields(mt_struct *mt, mt_dict):
	set_mt_struct_common_fields(mt, mt_dict)
	mt.aaa = uint32_from_python(mt_dict['aaa'])
	mt.maskB = uint32_from_python(mt_dict['maskB'])
	mt.maskC = uint32_from_python(mt_dict['maskC'])
	mt.i = uint32_from_python(mt_dict['i'])

cdef set_mt_struct_common_fields(mt_struct *mt, common_mt_dict):
	mt.mm = <int>common_mt_dict['mm']
	mt.nn = <int>common_mt_dict['nn']
	mt.rr = <int>common_mt_dict['rr']
	mt.ww = <int>common_mt_dict['ww']
	mt.wmask = uint32_from_python(common_mt_dict['wmask'])
	mt.umask = uint32_from_python(common_mt_dict['umask'])
	mt.lmask = uint32_from_python(common_mt_dict['lmask'])
	mt.shift0 = <int>common_mt_dict['shift0']
	mt.shift1 = <int>common_mt_dict['shift1']
	mt.shiftB = <int>common_mt_dict['shiftB']
	mt.shiftC = <int>common_mt_dict['shiftC']


cdef class DcmtRandomState:

	cdef mt_struct *mt

	def __init__(self, *args, wordlen=32, exponent=521, id=0, gen_seed=None):
		cdef int w, p, mid, sid
		validate_parameters(wordlen, exponent, id, id, &w, &p, &sid, &mid)
		cdef uint32_t s = get_seed(gen_seed)

		self.mt = get_mt_parameter_id_st(w, p, sid, s)
		self.seed(*args)

	def __dealloc__(self):
		if self.mt != NULL:
			free_mt_struct(self.mt)
			self.mt = NULL

	def __getstate__(self):
		return self.get_state()

	def __setstate__(self, state):
		self.set_state(state)

	def __reduce__(self):
		return self.__class__, (), self.get_state()

	def seed(self, seed=None):
		cdef uint32_t s = get_seed(seed)
		sgenrand(s, self.mt)

	def get_state(self):
		cdef ndarray state "arrayObject_state"
		cdef mt_struct *mt = <mt_struct *>self.mt
		state = <ndarray>numpy.empty(mt.nn, numpy.uint32)
		memcpy(<void*>(state.data), <void*>(mt.state), mt.nn * sizeof(uint32_t))
		return (1, get_mt_struct_fields(mt), state)

	def set_state(self, state):
		version, fields, state_vec = state
		assert version == 1, "State version " + str(version) + " is not supported"

		cdef mt_struct *mt = <mt_struct *>malloc(sizeof(mt_struct))
		cdef ndarray obj "arrayObject_obj"

		set_mt_struct_fields(mt, fields)

		mt.state = <uint32_t *>malloc(sizeof(uint32_t) * mt.nn)

		obj = <ndarray>PyArray_ContiguousFromObject(state_vec, NPY_UINT, 1, 1)
		if obj.dimensions[0] != mt.nn:
			raise ValueError("wrong state vector size")
		memcpy(<void*>(mt.state), <void*>(obj.data), mt.nn * sizeof(uint32_t))
		self.mt = mt

	def rand(self, *size):
		return self.random_sample(size=size)

	def random_sample(self, size=None):
		cdef double *array_data
		cdef ndarray array "arrayObject"
		cdef long length
		cdef long i

		if size is None:
			return random_float(self.mt)
		else:
			array = <ndarray>numpy.empty(size, numpy.float64)
			length = PyArray_SIZE(array)
			array_data = <double *>array.data
			for i in range(length):
				array_data[i] = random_float(self.mt)
			return array

	@classmethod
	def range(cls, *args, wordlen=32, exponent=521, gen_seed=None):
		cdef int i, count
		cdef mt_struct **mts = NULL

		res = create_mt_range(args, wordlen, exponent, gen_seed, &mts, &count)
		if res != None:
			return res

		cdef DcmtRandomState rng
		rngs = []
		for i in range(count):
			rng = <DcmtRandomState>cls.__new__(cls)
			rng.mt = mts[i]
			rng.seed()

			rngs.append(rng)

		# We do not need array of pointers anymore, but we are keeping pointers to mt_struct's
		# So we are not using free_mt_struct_array()
		free(mts)

		return rngs

	@classmethod
	def from_mt_range(cls, mt_common, mt_unique):

		cdef mt_struct *mt
		cdef DcmtRandomState rng
		cdef ndarray unique_fields "arrayObject_unique"
		unique_fields = <ndarray>mt_unique
		cdef uint32_t *data = <uint32_t *>unique_fields.data

		rngs = []
		for i in xrange(mt_unique.shape[0]):
			rng = <DcmtRandomState>cls.__new__(cls)

			mt = <mt_struct *>malloc(sizeof(mt_struct))

			set_mt_struct_common_fields(mt, mt_common)

			mt.aaa = data[i * 4 + 0]
			mt.maskB = data[i * 4 + 1]
			mt.maskC = data[i * 4 + 2]
			mt.i = <int>data[i * 4 + 3]

			mt.state = <uint32_t *>malloc(sizeof(uint32_t) * mt.nn)

			rng.mt = mt
			rng.seed()

			rngs.append(rng)

		return rngs


def mt_range(*args, wordlen=32, exponent=521, gen_seed=None):

	cdef int i, count
	cdef mt_struct **mts = NULL

	res = create_mt_range(args, wordlen, exponent, gen_seed, &mts, &count)
	if res != None:
		return res

	common_fields = get_mt_struct_fields(mts[0])

	cdef ndarray unique_fields "arrayObject_unique"
	unique_fields = <ndarray>numpy.empty((count, 4), numpy.uint32)

	cdef uint32_t *data = <uint32_t *>unique_fields.data
	for i in range(count):
		data[i * 4 + 0] = mts[i].aaa
		data[i * 4 + 1] = mts[i].maskB
		data[i * 4 + 2] = mts[i].maskC
		data[i * 4 + 3] = <uint32_t>mts[i].i

	free_mt_struct_array(mts, count)

	return common_fields, unique_fields



include "common.pxi"

from random import Random

from dcmt.exceptions import DcmtError, DcmtParameterError


cdef class RandomContainer:

	cdef mt_struct *mt

	def __init__(self):
		self.mt = NULL

	cdef void initWithStruct(self, mt_struct *mt):
		self.mt = mt

	cdef void initWithParams(self, int wordlen, int exponent, int id, uint32_t seed) except *:
		self.mt = get_mt_parameter_id_st(wordlen, exponent, id, seed)
		if self.mt == NULL:
			raise DcmtError("Failed to create RNG")

	def __dealloc__(self):
		if self.mt != NULL:
			free_mt_struct(self.mt)
			self.mt = NULL

	cdef seed(self, uint32_t seed):
		sgenrand(seed, self.mt)

	cdef random(self):
		return random_float(self.mt)

	cdef random_raw(self):
		return random_uint32(self.mt)

	def jumpahead(self, n):

		if type(n) not in (int, long):
			raise TypeError("jumpahead requires an integer, not " + str(type(n)))

		cdef uint32_t *state = self.mt.state
		cdef uint32_t i, j
		cdef uint32_t tmp
		cdef int nn = self.mt.nn

		for i in range(nn - 1, 1, -1):
			j = n % int(i)
			tmp = state[i]
			state[i] = state[j]
			state[j] = tmp

		for i in range(nn):
			state[i] += i + 1

		self.mt.i = nn

	def getrandbits(self, kk):
		# WARNING: currently works only for 31 and 32 bit randoms

		cdef int k = <int>kk, i, j, bytes
		cdef uint32_t r
		cdef unsigned char *bytearray

		if k <= 0:
			raise ValueError("number of bits must be greater than zero")

		bytes = ((k - 1) / 32 + 1) * 4
		bytearray = <unsigned char *>PyMem_Malloc(bytes);
		if bytearray == NULL:
			raise MemoryError()

	    # Fill-out whole words, byte-by-byte to avoid endianness issues
		cdef int bytelen = 4 if self.mt.ww == 32 else 3
		cdef int bitlen = bytelen * 8

		for i in range(0, bytes, bytelen):
			r = random_uint32(self.mt)
			if k < bitlen:
				r >>= (bitlen - k)

			for j in range(bytelen):
				bytearray[i+j] = <unsigned char>(r >> (8 * j))

			k -= bitlen

	    # little endian order to match bytearray assignment order
		result = _PyLong_FromByteArray(bytearray, bytes, 1, 0)
		PyMem_Free(bytearray)
		return result

	def getstate(self):
		"""Return internal state; can be passed to setstate() later."""
		cdef mt_struct *mt = self.mt

		state = []
		for i in range(self.mt.nn):
			state.append(mt.state[i])

		return (1, (mt.aaa, mt.mm, mt.nn, mt.rr, mt.ww,
			mt.wmask, mt.umask, mt.lmask,
			mt.shift0, mt.shift1, mt.shiftB, mt.shiftC,
			mt.maskB, mt.maskC, mt.i, tuple(state)))

	def setstate(self, state):
		"""Restore internal state from object returned by getstate()."""
		version, fields = state
		if version != 1:
			raise DcmtError("State version " + str(version) + " is not supported")

		cdef mt_struct *mt = <mt_struct *>malloc(sizeof(mt_struct))

		mt.aaa, mt.mm, mt.nn, mt.rr, mt.ww, \
			mt.wmask, mt.umask, mt.lmask, \
			mt.shift0, mt.shift1, mt.shiftB, mt.shiftC, \
			mt.maskB, mt.maskC, mt.i, state_vec = fields

		mt.state = <uint32_t *>malloc(sizeof(uint32_t) * mt.nn)
		for i in range(mt.nn):
			mt.state[i] = state_vec[i]

		self.mt = mt


class DcmtRandom(Random):

	def __init__(self, *args, wordlen=32, exponent=521, id=0, gen_seed=None):
		cdef int w, p, mid, sid
		cdef uint32_t s = get_seed(gen_seed)

		validate_parameters(wordlen, exponent, id, id, &w, &p, &sid, &mid)

		cdef RandomContainer rc = <RandomContainer>RandomContainer()
		rc.initWithParams(w, p, sid, s)
		self.rc = rc

		self.seed(*args)

	def seed(self, a=None):
		cdef uint32_t seed = get_seed(a)
		cdef RandomContainer rc = <RandomContainer>self.rc
		rc.seed(seed)

	def random(self):
		cdef RandomContainer rc = <RandomContainer>self.rc
		return rc.random()

	def random_raw(self):
		cdef RandomContainer rc = <RandomContainer>self.rc
		return rc.random_raw()

	def getstate(self):
		cdef RandomContainer rc = <RandomContainer>self.rc
		return rc.getstate()

	def setstate(self, state):
		cdef RandomContainer rc = <RandomContainer>self.rc
		rc.setstate(state)

	def getrandbits(self, k):
		cdef RandomContainer rc = <RandomContainer>self.rc
		return rc.getrandbits(k)

	def jumpahead(self, n):
		cdef RandomContainer rc = <RandomContainer>self.rc
		rc.jumpahead(n)

	@classmethod
	def range(cls, *args, wordlen=32, exponent=521, gen_seed=None):

		cdef int i, count
		cdef mt_struct **mts = NULL

		res = create_mt_range(args, wordlen, exponent, gen_seed, &mts, &count)
		if res != None:
			return res

		cdef RandomContainer rc
		rngs = []
		for i in range(count):
			rc = <RandomContainer>RandomContainer()
			rc.initWithStruct(mts[i])

			rng = cls.__new__(cls)
			cls.rc = rc
			rng.seed()

			rngs.append(rng)

		# We do not need array of pointers anymore, but we are keeping pointers to mt_struct's
		# So we are not using free_mt_struct_array()
		free(mts)

		return rngs



import os.path
import sys
major, minor, _, _, _ = sys.version_info
if not (major == 2 and minor >= 5):
	print("Python >=2.5 is required to use this module.")
	sys.exit(1)

try:
    from Cython.Distutils import build_ext
    has_cython = True
except:
    has_cython = False

try:
	from distutils.core import setup, Extension
except ImportError:
	from setuptools import setup, Extension
	# In order to work with Cython, setuptools needs a weird hack involving
	# fake Pyrex module. I do not really want to do that.
	has_cython = False

def get_symbol(path, symbol):
	full_path = os.path.join(setup_dir, *path)
	globals_dict = {}
	execfile(full_path, globals_dict)
	return globals_dict[symbol]


setup_dir = os.path.split(os.path.abspath(__file__))[0]
DOCUMENTATION = open(os.path.join(setup_dir, 'README.rst')).read()

VERSION = get_symbol(('dcmt', 'version.py'), 'VERSION_STRING')

class NumpyExtension(Extension):
	# nicked from
	# http://mail.python.org/pipermail/distutils-sig/2007-September/008253.html
	# solution by Michael Hoffmann
	def __init__(self, *args, **kwargs):
		Extension.__init__(self, *args, **kwargs)
		self._include_dirs = self.include_dirs
		del self.include_dirs # restore overwritten property

	def get_numpy_incpath(self):
		from imp import find_module
		# avoid actually importing numpy, it screws up distutils
		file, pathname, descr = find_module("numpy")
		from os.path import join
		return join(pathname, "core", "include")

	def get_include_dirs(self):
		return self._include_dirs + [self.get_numpy_incpath()]

	def set_include_dirs(self, value):
		self._include_dirs = value

	def del_include_dirs(self):
		pass

	include_dirs = property(get_include_dirs, set_include_dirs, del_include_dirs)


c_sources = [
	'src/dcmt/lib/check32.c',
	'src/dcmt/lib/eqdeg.c',
	'src/dcmt/lib/genmtrand.c',
	'src/dcmt/lib/init.c',
	'src/dcmt/lib/mt19937.c',
	'src/dcmt/lib/prescr.c',
	'src/dcmt/lib/seive.c',

	'src/wrapper/common.c',
]

pyrandom = Extension("dcmt.pyrandom",
	include_dirs = ['src/dcmt/include', 'src/dcmt/lib'],
	extra_compile_args = ['-Wall', '-Wmissing-prototypes', '-O3', '-std=c99'],
	sources = c_sources + ['src/wrapper/pyrandom.' + ('pyx' if has_cython else 'c')])

numpyrandom = NumpyExtension("dcmt.numpyrandom",
	include_dirs = ['src/dcmt/include', 'src/dcmt/lib'],
	extra_compile_args = ['-Wall', '-Wmissing-prototypes', '-O3', '-std=c99'],
	sources = c_sources + ['src/wrapper/numpyrandom.' + ('pyx' if has_cython else 'c')])

setup(
	name='dcmt',
	packages=['dcmt'],
	cmdclass={'build_ext': build_ext} if has_cython else {},
	ext_modules=[pyrandom, numpyrandom],
	requires=['numpy'],
	install_requires=['numpy'],
	version=VERSION,
	author='Bogdan Opanchuk',
	author_email='mantihor@gmail.com',
	url='http://github.com/Manticore/python-dcmt',
	description='Dynamical creation of Mersenne twisters',
	long_description=DOCUMENTATION,
	classifiers=[
		'Development Status :: 4 - Beta',
		'Intended Audience :: Developers',
		'License :: OSI Approved :: MIT License',
		'Operating System :: OS Independent',
		'Programming Language :: Python :: 2',
		'Topic :: Scientific/Engineering :: Mathematics'
	]
)
